/*
 * Copyright (C) 2005 - 2011, Roni Zaharia
 */

/// \page MTTA Multi-Threaded Test Applications
/// \section MultithreadedServerExample A C# Multi-Threaded DICOM Server
/// The C# test applications can be downloaded from www.roniza.com/downloads
/// Multithreaded Server Example
/// Waits for multiple connections simultaniously from the client

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using rzdcxLib;
using System.Runtime.InteropServices;
using System.Threading;


namespace MultithreadedServerExample
{
    /// \example MultithreadedServerExampleForm.cs
    /// This example demonstrates the use of the Acceptor in a multithreaded way.
    public partial class MultithreadedServerExampleForm : Form
    {
        public MultithreadedServerExampleForm()
        {
            InitializeComponent();
        }

        class AssocHandler
        {
            string _aeTitle;
            ushort _port;
            volatile static private int _filecount = 0;

            public AssocHandler(string aeTitle, ushort port)
            {
                _aeTitle = aeTitle;
                _port = port;
            }

            public void DoWork()
            {
                DCXACC acc = null;
                try
                { 
                    acc = CreateAccepter();

                    // This is the standard loop to wait for connections and commands
                    bool recievedConnection = false;
                    while (!recievedConnection)
                    {
                        if (acc.WaitForConnection(_aeTitle, _port, 5))
                        {
                            // Notify the main thread that a connection
                            // was established
                            lock (this)
                            {
                                Monitor.Pulse(this);
                            }

                            recievedConnection = true;

                            bool res;
                            do
                            {
                                // Wait for a command from the client
                                if (acc.WaitForCommand(180))
                                {
                                    res = true;
                                }
                                else
                                {
                                    res = false;
                                }
                            } while (res);
                        }
                    }
                }
                catch (Exception)
                {
                }
                finally
                {
                    if (acc != null)
                        Marshal.ReleaseComObject(acc);
                }
            }

            private DCXACC CreateAccepter()
            {
                // Create the acceptor object here
                DCXACC acc = new DCXACC();
                // Connect all the events to the callback methods
                acc.OnConnection += new IDCXACCEvents_OnConnectionEventHandler(OnConnectionEventHandler);
                acc.OnTimeout += new IDCXACCEvents_OnTimeoutEventHandler(OnTimeoutEventHandler);
                acc.OnCommitResult += new IDCXACCEvents_OnCommitResultEventHandler(OnCommitResultEventHandler);
                acc.OnStorageCommitRequest += new IDCXACCEvents_OnStorageCommitRequestEventHandler(OnStorageCommitRequestEventHandler);
                acc.OnStoreSetup += new IDCXACCEvents_OnStoreSetupEventHandler(OnStoreSetupEventHandler);
                acc.OnStoreDone += new IDCXACCEvents_OnStoreDoneEventHandler(OnStoreDoneEventHandler);
                return acc;
            }

            /* Callback method when a connection is recieved
           Here you can check the calling ae and accept or 
           reject the association*/
            private void OnConnectionEventHandler(string callingTitle,
                                           string calledTitle,
                                           string callingHost,
                                           ref bool acceptConnection)
            {
                acceptConnection = true;
            }

            /* When a store command is recieved, you can provide the
            filename to store to*/
            private void OnStoreSetupEventHandler(ref string filename)
            {
                filename = ".\\dcm" + (++_filecount) + ".dcm";

                MessageBox.Show("Store setup on file: " + filename);
            }

            /* When store is done, handle the file here and respond if storage went well */
            private void OnStoreDoneEventHandler(string filename, bool status, ref bool accept)
            {
                MessageBox.Show("Store done, file: " + filename + " status: " + status);

                accept = true;
            }

            /* Handle the on timeout event here */
            private void OnTimeoutEventHandler()
            {
                MessageBox.Show("A timeout occurred");
            }

            /* When a result for the storage commit is recieved 
               this event occurs. Handle the succeeded and failed instances here*/
            private void OnCommitResultEventHandler(bool status,
                                                    string transactionUID,
                                                    string succeededInstances,
                                                    string failedInstances)
            {
                MessageBox.Show("Commit result: Status: " + status + "\n" +
                                "TransactionUID: " + transactionUID + "\n" +
                                "SucceededInstances: " + succeededInstances + "\n" +
                                "FailedInstances: " + failedInstances);
            }

            /* When a request for the storage commit is recieved 
               this event occurs.*/
            private void OnStorageCommitRequestEventHandler(string transactionUID,
                                                    string instances,
                                                    ref bool acceptStorageCommit)
            {
                MessageBox.Show("Commit Request: " +
                                "TransactionUID: " + transactionUID + "\n" +
                                "Instances: " + instances
                                );

                acceptStorageCommit = true;
            }
        }

        private void ListenBtn_Click(object sender, EventArgs e)
        {
            DCXAPP app = new DCXAPP();
            app.StartLogging("c:\\rzdcxLog.txt");

            // This is the standard loop to wait for connections and commands
            while (true)
            {
                AssocHandler handler = new AssocHandler(LocalAEEdit.Text, ushort.Parse(PortEdit.Text));

                Thread t = new Thread(new ThreadStart(handler.DoWork));
                t.SetApartmentState(ApartmentState.MTA);
                t.Start();

                // Wait until the thread tells us that a connection was established
                // so we can create a new thread and wait for another connection
                lock (handler)
                {
                    Monitor.Wait(handler);
                }
            }

            app.StopLogging();

            ReleaseComObject(app);
        }

        /* This has to be called for all the com objects to release the
           memory!!! */
        private void ReleaseComObject(object o)
        {
            if (o != null)
                Marshal.ReleaseComObject(o);
        }
    }
}